`timescale 1ns / 1ps
`define TRUE 1'b1
`define FALSE 1'b0
`define CRVENO 2'd0
`define ZUTO 2'd1
`define ZELENO 2'd2
// definicije stanja: glavni  sporedni
`define S0 3'd0  //   zeleno  crveno
`define S1 3'd1  //    zuto   crveno
`define S2 3'd2  //   crveno  crveno
`define S3 3'd3  //   crveno  zeleno
`define S4 3'd4  //   crveno   zuto
// kasnjenja
`define ZUTO_CRVENO 3
`define CRVENO_ZELENO 2

module semafor(glavni, sporedni, X, clock, clear);
output [1:0] glavni, sporedni; // 3 mogua svijetla (C,,Z)
reg [1:0] glavni, sporedni;
input X; // ako je TRUE ima vozila na sporednom putu
input clock, clear;

// pomocne promjenljive (za stanja automata)
reg [2:0] stanje;
reg [2:0] sljedece_stanje;

// semafor poinje u stanju S0
initial // inicijalizacija
begin
	stanje = `S0;
	sljedece_stanje = `S0;
	glavni = `ZELENO;
	sporedni = `CRVENO;
end

// promjena stanja se vri na pozitivnoj ivici takta
always @(posedge clock)
	stanje = sljedece_stanje;

// Izraunati vrijednost svjetlosnih signala
always @(stanje)
begin
	case(stanje)
		`S0: 	begin
					glavni = `ZELENO;
					sporedni = `CRVENO;
				end
		`S1: 	begin
					glavni = `ZUTO;
					sporedni = `CRVENO;
				end
		`S2: 	begin
					glavni = `CRVENO;
					sporedni = `CRVENO;
				end
		`S3: 	begin
					glavni = `CRVENO;
					sporedni = `ZELENO;
				end
		`S4: 	begin
					glavni = `CRVENO;
					sporedni = `ZUTO;
				end
	endcase
end

// automat
always @(stanje or clear or X)
begin
	if (clear)
		sljedece_stanje = `S0;
	else
		case (stanje)
			`S0: 	if(X) sljedece_stanje = `S1;
					else sljedece_stanje = `S0;
			`S1: 	begin // saekaj nekoliko pozitivnih ivica takta
						repeat(`ZUTO_CRVENO) @(posedge clock);
						sljedece_stanje = `S2;
					end
			`S2: 	begin // saekaj nekoliko pozitivnih ivica takta
						repeat(`CRVENO_ZELENO) @(posedge clock) ;
						sljedece_stanje = `S3;
					end
			`S3: 	if(X) sljedece_stanje = `S3;
					else sljedece_stanje = `S4;
			`S4: 	begin // saekaj nekoliko pozitivnih ivica takta
						repeat(`ZUTO_CRVENO) @(posedge clock) ;
						sljedece_stanje = `S0;
					end
			default: sljedece_stanje = `S0;
		endcase
end
endmodule		

module stimulus;
wire [1:0] GLAVNI, SPOREDNI;
reg VOZILO_NA_SPOREDNOM;
reg CLOCK, CLEAR;

// instanciranje kontrolera
semafor S(GLAVNI, SPOREDNI, VOZILO_NA_SPOREDNOM, CLOCK, CLEAR);

initial
	$monitor ($time, " Glavni = %b Sporedni = %b X = %b",
		GLAVNI, SPOREDNI, VOZILO_NA_SPOREDNOM);

initial
begin
	CLOCK = `FALSE;
	forever #5 CLOCK = ~CLOCK;
end

initial // resetovanje kontrolera
begin
	CLEAR = `TRUE;
	repeat (5) @(negedge CLOCK);
	CLEAR = `FALSE;
end

// stimulus
initial
begin
	VOZILO_NA_SPOREDNOM = `FALSE;
	#200 VOZILO_NA_SPOREDNOM = `TRUE;
	#100 VOZILO_NA_SPOREDNOM = `FALSE;
	#200 VOZILO_NA_SPOREDNOM = `TRUE;
	#100 VOZILO_NA_SPOREDNOM = `FALSE;
	#200 VOZILO_NA_SPOREDNOM = `TRUE;
	#100 VOZILO_NA_SPOREDNOM = `FALSE;
	#100 $finish;
end
endmodule
